EBB_Bar = AceLibrary("AceOO-2.0").Class()

function EBB_Bar:ToString()
	return "EBB_Bar"
end

function EBB_Bar.prototype:init(arg)
	EBB_Bar.super.prototype.init(self)
	self.frames = {}
end

function EBB_Bar.prototype:Reset()
	self.frames.container:Hide()
	self.layout = nil
	self.data = nil
	self:SetParent(UIParent)
end

function EBB_Bar.prototype:GetContainer()
	return self.frames.container
end

function EBB_Bar.prototype:SetParent(parent)
	if self.frames.container then
		self.frames.container:SetParent(parent)
		if self.frames.bar then
			self.frames.container:SetFrameLevel(self.frames.bar:GetFrameLevel() + 1) -- fix frame order
		end
	end
end

function EBB_Bar.prototype:UpdateLayout(layout)
	if layout then
		self.layout = layout
	else
		layout = self.layout
		if not layout then
			return
		end
	end
	
	local frames = self.frames
	
-- container
	if not frames.container then
		frames.container = CreateFrame("button", nil, UIParent)
		frames.container:SetFrameStrata("BACKGROUND")
	end
	frames.container:SetHeight(layout.height)
	frames.container:SetWidth(layout.width)
	local leftoffset = 0
	local rightoffset = 0

-- icon
	if layout.icon then
		if not frames.icon then
			frames.icon = frames.container:CreateTexture(nil, "ARTWORK")
		end
		frames.icon:ClearAllPoints()
		frames.icon:SetHeight(layout.height)
		frames.icon:SetWidth(layout.height)
		if layout.icon == "LEFT" then
			leftoffset = layout.height
			frames.icon:SetPoint("LEFT", frames.container)
			
		end
		if layout.icon == "RIGHT" then
			rightoffset = -layout.height
			frames.icon:SetPoint("RIGHT", frames.container)
			
		end
		frames.icon:Show()
	else
		if frames.icon then frames.icon:Hide() end
	end

-- iconcount
	if layout.icon and layout.iconcount then
		if not frames.iconcount then
			frames.iconcount = frames.container:CreateFontString(nil, "OVERLAY")
		end
		frames.iconcount:ClearAllPoints()
		frames.iconcount:SetPoint(layout.iconcountanchor, frames.icon, layout.iconcountanchor, (string.find(layout.iconcountanchor, "LEFT") and 3) or (string.find(layout.iconcountanchor, "RIGHT") and -3) or 0, (string.find(layout.iconcountanchor, "TOP") and -3) or (string.find(layout.iconcountanchor, "BOTTOM") and 3) or 0)
		frames.iconcount:SetFont(layout.iconcountfont, layout.iconcountfontsize)
		frames.iconcount:Show()
	else
		if frames.iconcount then frames.iconcount:Hide() end
	end

-- iconborder
	if layout.icon then
		if not frames.iconborder then
			frames.iconborder = frames.container:CreateTexture(nil, "OVERLAY")
		end
		frames.iconborder:ClearAllPoints()
		frames.iconborder:SetPoint("TOPLEFT", frames.icon)
		frames.iconborder:SetPoint("BOTTOMRIGHT", frames.icon)
		frames.iconborder:Show()
	else
		if frames.iconborder then frames.iconborder:Hide() end
	end

-- bar
	if layout.bar then
		if not frames.bgbar then
			frames.bgbar = CreateFrame("StatusBar", nil, frames.container)
		end
		frames.bgbar:ClearAllPoints()
		frames.bgbar:SetPoint("TOPLEFT", frames.container, "TOPLEFT", leftoffset, 0)
		frames.bgbar:SetPoint("BOTTOMRIGHT", frames.container, "BOTTOMRIGHT", rightoffset, 0)
		frames.bgbar:SetFrameStrata("BACKGROUND")
		frames.bgbar:SetFrameLevel(frames.container:GetFrameLevel() + 1)
		frames.bgbar:SetStatusBarTexture(layout.bar)
		frames.bgbar:SetMinMaxValues(0, 1)
		frames.bgbar:SetValue(1)
		frames.bgbar:Show()

		if not frames.bar then
			frames.bar = CreateFrame("StatusBar", nil, frames.container)
		end
		frames.bar:ClearAllPoints()
		frames.bar:SetPoint("TOPLEFT", frames.container, "TOPLEFT", leftoffset, 0)
		frames.bar:SetPoint("BOTTOMRIGHT", frames.container, "BOTTOMRIGHT", rightoffset, 0)
		frames.bar:SetFrameStrata("BACKGROUND")
		frames.bar:SetFrameLevel(frames.bgbar:GetFrameLevel() + 1)
		frames.bar:SetStatusBarTexture(layout.bar)
		frames.bar:Show()

		frames.container:SetFrameLevel(frames.bar:GetFrameLevel() + 1) -- fix frame order
	else
		if frames.bgbar then frames.bgbar:Hide() end
		if frames.bar then frames.bar:Hide() end
	end

-- textTL
	if layout.textTL then
		if not frames.textTL then
			frames.textTL = frames.container:CreateFontString(nil, "OVERLAY")
		end
		frames.textTL:ClearAllPoints()
		frames.textTL:SetPoint("TOPLEFT", frames.container, "TOPLEFT", leftoffset + 2, -2)
		frames.textTL:SetFontObject(GameFontNormal)
		frames.textTL:SetFont(layout.textTLfont, layout.textTLfontsize)
		frames.textTL:SetTextColor(layout.textTLcolor[1], layout.textTLcolor[2], layout.textTLcolor[3], 1)
		frames.textTL:SetJustifyH((layout.textTR and "LEFT") or layout.textTLalign)
		if not layout.textBL then
			frames.textTL:SetPoint("BOTTOMLEFT", frames.container, "BOTTOMLEFT", leftoffset + 2, 2)
			frames.textTL:SetJustifyV("CENTER")
		else
			frames.textTL:SetJustifyV("TOP")
		end
		frames.textTL:Show()
	else
		if frames.textTL then frames.textTL:Hide() end
	end

-- textTR
	if layout.textTR then
		if not frames.textTR then
			frames.textTR = frames.container:CreateFontString(nil, "OVERLAY")
		end
		frames.textTR:ClearAllPoints()
		frames.textTR:SetPoint("TOPRIGHT", frames.container, "TOPRIGHT", rightoffset - 2, -2)
		frames.textTR:SetFontObject(GameFontNormal)
		frames.textTR:SetFont(layout.textTRfont, layout.textTRfontsize)
		frames.textTR:SetTextColor(layout.textTRcolor[1], layout.textTRcolor[2], layout.textTRcolor[3], 1)
		frames.textTR:SetJustifyH("RIGHT")
		if not layout.textBL then
			frames.textTR:SetPoint("BOTTOMRIGHT", frames.container, "BOTTOMRIGHT", rightoffset - 2, 2)
			frames.textTR:SetJustifyV("CENTER")
		else
			frames.textTR:SetJustifyV("TOP")
		end
		frames.textTR:Show()
	else
		if frames.textTR then frames.textTR:Hide() end
	end

-- textBL
	if layout.textTL and layout.textBL then
		if not frames.textBL then
			frames.textBL = frames.container:CreateFontString(nil, "OVERLAY")
		end
		frames.textBL:ClearAllPoints()
		frames.textBL:SetPoint("BOTTOMLEFT", frames.container, "BOTTOMLEFT", leftoffset + 2, 2)
		frames.textBL:SetFontObject(GameFontNormal)
		frames.textBL:SetFont(layout.textBLfont, layout.textBLfontsize)
		frames.textBL:SetTextColor(layout.textBLcolor[1], layout.textBLcolor[2], layout.textBLcolor[3], 1)
		frames.textBL:SetJustifyH((layout.textBR and "LEFT") or layout.textBLalign)
		frames.textBL:Show()
	else
		if frames.textBL then frames.textBL:Hide() end
	end

-- textTR
	if layout.textTL and layout.textBR then
		if not frames.textBR then
			frames.textBR = frames.container:CreateFontString(nil, "OVERLAY")
		end
		frames.textBR:ClearAllPoints()
		frames.textBR:SetPoint("BOTTOMRIGHT", frames.container, "BOTTOMRIGHT", rightoffset - 2, 2)
		frames.textBR:SetFontObject(GameFontNormal)
		frames.textBR:SetFont(layout.textBRfont, layout.textBRfontsize)
		frames.textBR:SetTextColor(layout.textBRcolor[1], layout.textBRcolor[2], layout.textBRcolor[3], 1)
		frames.textBR:SetJustifyH("RIGHT")
		frames.textBR:Show()
	else
		if frames.textBR then frames.textBR:Hide() end
	end
end

function EBB_Bar.prototype:UpdateData(data)
	if data then
		self.data = data
	else
		data = self.data
		if not data then
			return
		end
	end

	local frames = self.frames

	if self.layout.icon then
		frames.icon:SetTexture(data.icon)
		if data.type == "DEBUFF" then
			frames.iconborder:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")
			frames.iconborder:SetTexCoord(0.296875, 0.5703125, 0, 0.515625)
			local debuffcolor
			if ( data.debufftype ) then
				debuffcolor = DebuffTypeColor[data.debufftype]
			else
				debuffcolor = DebuffTypeColor["none"]
			end
			frames.iconborder:SetVertexColor(debuffcolor.r, debuffcolor.g, debuffcolor.b)
			frames.iconborder:Show()
		elseif data.type == "TENCH" then
			frames.iconborder:SetTexture("Interface\\Buttons\\UI-TempEnchant-Border")
			frames.iconborder:SetTexCoord(0, 1, 0, 1)
			frames.iconborder:Show()
		else
			frames.iconborder:Hide()
		end
	end
	if self.layout.iconcount then
		if data.charges > 1 then
			frames.iconcount:SetText(data.charges)
			frames.iconcount:Show()
		else
			frames.iconcount:Hide()
		end
	end
	if self.layout.bar then
		if data.untilcancelled then
			frames.bar:SetMinMaxValues(0, 1)
			frames.bar:SetValue(0)
		else
			frames.bar:SetMinMaxValues(0, data.timemax)
			frames.bar:SetValue(data.timeleft)
		end
		local color = self.layout["barcolor"]
		if data.type == "DEBUFF" and self.layout.debufftypecolor then
			local debuffcolor
			if ( data.debufftype ) then
				debuffcolor = DebuffTypeColor[data.debufftype]
			else
				debuffcolor = DebuffTypeColor["none"]
			end
			color = {debuffcolor.r, debuffcolor.g, debuffcolor.b}
		end
		frames.bar:SetStatusBarColor(color[1], color[2], color[3], 0.8)
		color = self.layout["barbgcolor"]
		frames.bgbar:SetStatusBarColor(color[1], color[2], color[3], 0.3)
	end
	if self.layout.textTL then
		frames.textTL:SetText(self:GetDataString(self.layout.textTL))
	end
	if self.layout.textTR then
		frames.textTR:SetText(self:GetDataString(self.layout.textTR))
	end
	if self.layout.textTL and self.layout.textBL then
		frames.textBL:SetText(self:GetDataString(self.layout.textBL))
	end
	if self.layout.textTL and self.layout.textBR then
		frames.textBR:SetText(self:GetDataString(self.layout.textBR))
	end
end

local romandigits = {	{["r"] = "M",  ["a"] = 1000},
						{["r"] = "CM", ["a"] =  900},
						{["r"] = "D",  ["a"] =  500},
						{["r"] = "CD", ["a"] =  400},
						{["r"] = "C",  ["a"] =  100},
						{["r"] = "XC", ["a"] =   90},
						{["r"] = "L",  ["a"] =   50},
						{["r"] = "XL", ["a"] =   40},
						{["r"] = "X",  ["a"] =   10},
						{["r"] = "IX", ["a"] =    9},
						{["r"] = "V",  ["a"] =    5},
						{["r"] = "IV", ["a"] =    4},
						{["r"] = "I",  ["a"] =    1}
					}

local function arabic_to_roman(arabic)
	local roman = ""
	for i = 1, #romandigits do
		while arabic >= romandigits[i].a do
			arabic = arabic - romandigits[i].a
			roman = roman..romandigits[i].r
		end
	end
	return roman
end

function EBB_Bar.prototype:GetDataString(datatype)
	if datatype == "NAME" then return self.data.name end
	if datatype == "NAMERANK" then return self.data.name..(self.data.rank and (" "..arabic_to_roman(self.data.rank)) or "") end
	if datatype == "NAMECOUNT" then return self.data.name..((self.data.charges > 1) and " x"..self.data.charges or "") end
	if datatype == "NAMERANKCOUNT" then return self.data.name..(self.data.rank and (" "..arabic_to_roman(self.data.rank)) or "")..((self.data.charges > 1) and " x"..self.data.charges or "") end
	if datatype == "RANK" then return self.data.rank and arabic_to_roman(self.data.rank) or "" end
	if datatype == "COUNT" then return ((self.data.charges > 1) and "x"..self.data.charges or "") end
	if datatype == "TIMELEFT" then return self:GetTimeString(self.data.timeleft, self.layout.timeformat) end
	if datatype == "DEBUFFTYPE" then return self.data.debufftype end
	return "???"
end

function EBB_Bar.prototype:GetTimeString(time, format)
	if self.data.untilcancelled then return "" end
	local abacus = AceLibrary("Abacus-2.0")
	if format == "DEFAULT" then return SecondsToTimeAbbrev(time) end
	if format == "EXTENDED" then return abacus:FormatDurationExtended(time) end
	if format == "FULL" then return abacus:FormatDurationFull(time) end
	if format == "SHORT" then return abacus:FormatDurationShort(time) end
	if format == "CONDENSED" then return abacus:FormatDurationCondensed(time) end
	return "???"
end

function EBB_Bar.prototype:UpdateTimeleft()
	if self.layout.bar and not self.data.untilcancelled then
		self.frames.bar:SetValue(self.data.timeleft)
	end
	if self.layout.textTL == "TIMELEFT" then
		self.frames.textTL:SetText(self:GetDataString("TIMELEFT"))
	end
	if self.layout.textTR == "TIMELEFT" then
		self.frames.textTR:SetText(self:GetDataString("TIMELEFT"))
	end
	if self.layout.textTL and self.layout.textBL == "TIMELEFT" then
		self.frames.textBL:SetText(self:GetDataString("TIMELEFT"))
	end
	if self.layout.textTL and self.layout.textBR == "TIMELEFT" then
		self.frames.textBR:SetText(self:GetDataString("TIMELEFT"))
	end
end
